package GGI::GraphType;

=head1 NAME

GGI::GraphType - Perl interface to LibGGI ggi_graphtype values

=head1 SYNOPSIS

  use GGI::GraphType;

  $g = new GGI::Graphtype;

  $g->depth(4);
  $depth = $g->depth;

  $g->size(16);
  $size = $g->size;
 
  $g->scheme(GT_TEXT);
  $scheme = $g->scheme;

  $g->subscheme(GT_AUTO);
  $subscheme = $g->subscheme;

  $g->graphtype(GT_TEXT16);
  $graphtype = $g->graphtype;

=head1 DESCRIPTION

Provides a simple interface into the subfields of the ggi_graphtype type in 
LibGGI.

=cut

use strict;
use Carp;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $AUTOLOAD);

require Exporter;
require DynaLoader;
require AutoLoader;


=head1 Exported constants

  GT_15BIT
  GT_16BIT
  GT_1BIT
  GT_24BIT
  GT_2BIT
  GT_32BIT
  GT_4BIT
  GT_8BIT
  GT_AUTO
  GT_GREYSCALE
  GT_INVALID
  GT_PALETTE
  GT_STATIC_PALETTE
  GT_SUB_HIGHBIT_RIGHT
  GT_SUB_PACKED_GETPUT
  GT_SUB_REVERSE_ENDIAN
  GT_TEXT
  GT_TEXT16
  GT_TEXT32
  GT_TRUECOLOR

=cut


@ISA = qw(Exporter DynaLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
	GT_15BIT
	GT_16BIT
	GT_1BIT
	GT_24BIT
	GT_2BIT
	GT_32BIT
	GT_4BIT
	GT_8BIT
	GT_AUTO
	GT_GREYSCALE
	GT_INVALID
	GT_PALETTE
	GT_STATIC_PALETTE
	GT_SUB_HIGHBIT_RIGHT
	GT_SUB_PACKED_GETPUT
	GT_SUB_REVERSE_ENDIAN
	GT_TEXT
	GT_TEXT16
	GT_TEXT32
	GT_TRUECOLOR
);
$VERSION = '0.03';

sub AUTOLOAD {
    my $constname;
    ($constname = $AUTOLOAD) =~ s/.*:://;
    my $val = constant($constname, @_ ? $_[0] : 0);
    if ($! != 0) {
	if ($! =~ /Invalid/) {
	    $AutoLoader::AUTOLOAD = $AUTOLOAD;
	    goto &AutoLoader::AUTOLOAD;
	}
	else {
		croak "Your vendor has not defined GGI::GraphType macro $constname";
	}
    }
    eval "sub $AUTOLOAD { $val }";
    goto &$AUTOLOAD;
}

bootstrap GGI::GraphType $VERSION;

# Preloaded methods

sub new {
    my ($class) = shift();
    my $this = {};
    $this->{did_malloc} = 1;
    $this->{obj} = ggi_graphtype_malloc();
    bless $this, $class;
    return $this;
}

sub not_quite_new {
    my ($class) = shift();
    my $this = {};
    $this->{obj} = shift();
    bless $this, $class;
    return $this;
}

sub depth {
    my $this = shift();
    if (@_ + 0) {
	return ggi_graphtype_set_field_depth($this->{obj}, shift());
    }
    return ggi_graphtype_get_field_depth($this->{obj});
}

sub size {
    my $this = shift();
    if (@_ + 0) {
	return ggi_graphtype_set_field_size($this->{obj}, shift());
    }
    return ggi_graphtype_get_field_size($this->{obj});
}

sub subscheme {
    my $this = shift();
    if (@_ + 0) {
	return ggi_graphtype_set_field_subscheme($this->{obj}, shift());
    }
    return ggi_graphtype_get_field_subscheme($this->{obj});
}

sub scheme {
    my $this = shift();
    if (@_ + 0) {
	return ggi_graphtype_set_field_scheme($this->{obj}, shift());
    }
    return ggi_graphtype_get_field_scheme($this->{obj});
}

sub graphtype {
    my $this = shift();
    if (@_ + 0) {
	return ggi_graphtype_set_all_fields($this->{obj}, shift());
    }
    return ggi_graphtype_get_all_fields($this->{obj});
}

sub DESTROY {
    my $this = shift();
    if ($this->{did_malloc}) {
	ggi_coord_free($this->{obj});
    }
    $this->{obj} = undef();
}

1;
__END__

=head1 AUTHOR

Brian S. Julin <bri@calyx.com>

=head1 SEE ALSO

GGI(3pm), perl(1).

=cut
